using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using static MenuObject;

public class LevelManager : MonoBehaviour
{
	[SerializeField] private Text complete;
	[SerializeField] private Text skin;
	[SerializeField] private Text levelNumb;
	[SerializeField] private Image spriteLevel;

	[SerializeField] private List<Skin> skins = new List<Skin>();
	[SerializeField] private List<LevelObject> levels = new List<LevelObject>();

	private GameManager gameManager;
	private MenuObject menuObject;

	private int indexCompletedLevel = 0;
	private int indexCurrentLevel = 0;
	private int activeSkinInd = 0;
	private GameObject currentLevel = null;

	private void Start()
	{ 
		gameManager = GameManager.instance;
		menuObject = gameManager.menuObject;

		indexCompletedLevel = levels.FindIndex(x => x.GetCompleted() == false);
		if(indexCompletedLevel < 0)
		{
			indexCompletedLevel = levels.Count - 1;
		}	

		indexCurrentLevel = indexCompletedLevel;
		DisplayLevelText();

		EventDispatcher.Add(EventNames.WinGame, Completed);
		EventDispatcher.Add(EventNames.OpenLevelMenu, DisplayLevelText);
		EventDispatcher.Add(EventNames.ExitPlay, ExitPlay); 
	}

	private void ExitPlay()
	{
		if (currentLevel != null)
		{
			Destroy(currentLevel);
		}
	}

	private void Completed()
	{
		if(currentLevel != null)
		{
			Destroy(currentLevel);
		}

		if(indexCurrentLevel == indexCompletedLevel)
		{
			levels[indexCompletedLevel].SaveCompleted();
			if (indexCompletedLevel < levels.Count - 1)
			{
				indexCompletedLevel++;
				indexCurrentLevel = indexCompletedLevel;
			}
		}
	}

	public void PreviousLevel()
	{
		SoundManager.ButtonSound();
		if(indexCurrentLevel > 0)
		{
			indexCurrentLevel--;
		}
		else
		{
			indexCurrentLevel = levels.Count - 1;
		}
		DisplayLevelText();
	}

	public void NextLevel()
	{
		SoundManager.ButtonSound();
		if (indexCurrentLevel < levels.Count - 1)
		{
			indexCurrentLevel++;
		}
		else
		{ 
			indexCurrentLevel = 0;
		}
		DisplayLevelText();
	}
	private void DisplayLevelText()
	{
		levelNumb.text = "Level: " + (indexCurrentLevel + 1);
		complete.text = levels[indexCurrentLevel].GetCompleted() == true ? "Completed" : "No Completed";
		var indSkin = skins.FindIndex(x => x.skin == levels[indexCurrentLevel].skinLevel);
		
		//activeSkinInd = indSkin;
		skin.text = skins[indSkin].GetName();
		spriteLevel.sprite = skins[indSkin].GetSprite();
		ChangeSkinTo(skins[indSkin].skin);
	}

	public void InstantiateLevel() 
	{
		SoundManager.ButtonSound();
		if (indexCurrentLevel <= indexCompletedLevel)
		{
			currentLevel = Instantiate(levels[indexCurrentLevel].gameObject);
			menuObject.ActivateMenu(TypeMenu.Game);
			var paramsLevel = levels[indexCurrentLevel].GetLevelSettings();
			EventDispatcher.SendEvent(EventNames.StartGame, paramsLevel.Item1, paramsLevel.Item2, paramsLevel.Item3);
		}
		else
		{
			var levelToNeed = (indexCompletedLevel + 1).ToString();
			EventDispatcher.SendEvent(EventNames.SetText, string.Format("  {0} ", levelToNeed) );//, 3f);
		} 
	}

	//skin
	private void DisableAll()
	{
		foreach (var item in skins)
		{
			item.Disable();
		}
	}

	public void ChangeSkinTo(Skins skin)
	{
		skins[activeSkinInd].Disable();
		activeSkinInd = skins.FindIndex(x => x.skin == skin);
		skins[activeSkinInd].Enable();
	}
	////
	///
}
