﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;
using static MenuObject;
using static SoundManager;

public class MenuController : MonoBehaviour
{
	private GameManager gameManager;
	private MenuObject menuObject;

	private void Awake()
	{
		gameManager = GameManager.instance;
		menuObject = gameManager.menuObject;
	}

	private void Start()
	{
		//menuObject.GetMenu<WinGame>(TypeMenu.WinGame).Init(); //init win game if this menu enabled in start
		EventDispatcher.Add(EventNames.WinGame, () =>
		{
			menuObject.ActivateMenu(TypeMenu.LevelMenu);
		});

	}

	public void Pause()
	{
		ButtonSound();
		menuObject.ActivateMenu(TypeMenu.Pause);
		EventDispatcher.SendEvent(EventNames.Pause);
	}

	public void MainMenu()
	{
		ButtonSound();
		menuObject.ActivateMenu(TypeMenu.MainMenu);
	}

	public void ExitPlay()
	{
		EventDispatcher.SendEvent(EventNames.ExitPlay);
		MainMenu();
	}

	public void LevelMenu()
	{
		ButtonSound();
		menuObject.ActivateMenu(TypeMenu.LevelMenu);
	}

	public void Play()
	{
		ButtonSound();
		menuObject.ActivateMenu(TypeMenu.Game);
	}

	public void DeactivateActive()
	{
		menuObject.DeactivateActive();
	}

	public void Exit()
	{
		Application.Quit();
	}

	public IEnumerator ExecuteAfterTime(float time, Action task)
	{
		yield return new WaitForSeconds(time);
		task();
	}

}


