﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEditor;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.Events;

//[DefaultExecutionOrder(-1)]
public class MenuObject : MonoBehaviour
{
	[SerializeField]
	public List<PartMenu> allMenus = new List<PartMenu>();
	private PartMenu activeMenu;
	private PartMenu previousMenu;

	private void Awake()
	{
		activeMenu = allMenus[0];// active menu is main menu
	}

	public void BackMenu()
	{
		activeMenu.Deactive();
		previousMenu.Active();
		activeMenu = previousMenu;
	}


	public PartMenu ActivateMenu(TypeMenu findeMenu)
	{
		previousMenu = activeMenu;

		activeMenu.Deactive();

		var thisMenu = allMenus.Find(x => x.typeMenu.CompareTo(findeMenu) == 0);
		thisMenu.Active();
		activeMenu = thisMenu;
		return activeMenu;
	}


	public void DeactivateActive()
    {
		activeMenu.Deactive();
	}

	public PartMenu GetMenu(TypeMenu findeMenu)
	{
		var thisMenu = allMenus.Find(x => x.typeMenu.CompareTo(findeMenu) == 0);
		return activeMenu;
	}

	public Q GetMenu<Q>(TypeMenu findeMenu)
		where Q: class
	{
		var thisMenu = allMenus.Find(x => x.typeMenu.CompareTo(findeMenu) == 0) as Q;
		return thisMenu;
	}

	public Button GetButon<Q>(Q typers)
		where Q : struct, IComparable
	{
		var menu = allMenus.Find(x => x is Menu<Q>);
		//Debug.Log((menu as T).typeMenu.ToString()); 
		//DebugNull(menu);
		var button = (menu as Menu<Q>).menuButton.Find(x => x.typeButton.CompareTo(typers) == 0);
		return button.currentButton;
	}

	public MenuButton<T> GetButonMenu<T>(T typers)
		where T : struct, IComparable
	{
		var menu = allMenus.Find(x => x is Menu<T>);
		var button = (menu as Menu<T>).menuButton.Find(x => x.typeButton.CompareTo(typers) == 0);
		return button;
	}


	//////////////////////////
	[Serializable]
	public class PartMenu : MonoBehaviour
	{
		public TypeMenu typeMenu;
		public GameObject menu;

		public void Active()
		{
			menu.SetActive(true);
		}

		public void Deactive()
		{
			menu.SetActive(false);
		}
	}

	[Serializable]
	public class Menu<T> : PartMenu where T : IComparable
	{
		public List<MenuButton<T>> menuButton;

		public MenuButton<T> GetMenuButton(T typeButton)
		{
			return menuButton.Find(x => x.typeButton.CompareTo(typeButton) == 0);
		}

		public Button GetButton(T typeButton)
		{
			return menuButton.Find(x => x.typeButton.CompareTo(typeButton) == 0).currentButton;
		}

		public void SetListner(T typeButton, UnityAction task)
		{
			Button findeButton =  menuButton.Find(x => x.typeButton.CompareTo(typeButton) == 0).currentButton;
			if (findeButton == null) Debug.LogError("not found button");
			findeButton.onClick.RemoveAllListeners();
			findeButton.onClick.AddListener(task);
		}


		#region ContextComand
		public virtual void FindeMenuButtons()
		{
			menu = gameObject;
			menuButton.Clear();
			SetMenuButtons(this);
		}

		public void SetMenuButtons<T>(Menu<T> thisMenu)
			where T : notnull, IComparable		{
			var allButMenu = thisMenu.menu.GetComponentsInChildren<Button>(); //true

			for (int i = 0; i < allButMenu.Length; i++)
			{
				thisMenu.menuButton.Add(new MenuButton<T>());
				thisMenu.menuButton[i].currentButton = allButMenu[i];
				thisMenu.menuButton[i].Name = allButMenu[i].name;
				thisMenu.menuButton[i].imageButton = allButMenu[i].gameObject.GetComponent<Image>();
				thisMenu.menuButton[i].textButton = allButMenu[i].gameObject.GetComponentInChildren<Text>();
			}
		}
		#endregion
	}

	public enum TypeMenu
	{
		MainMenu,
		Game,
		LevelMenu,
		Pause,
	}


	[Serializable]
	public class MenuButton<T> where T : IComparable //settings button
	{
		public string Name;
		public Button currentButton;
		public T typeButton;
		public Text textButton;
		public Image imageButton;
		public List<Sprite> swithSprite = new List<Sprite>(); // on 0 ; off 1
		public bool onOffSprite = true;

		public void SetName(string name)
		{
			Name = name;
		}
		public void SetInteractable(bool interactables)
		{
			currentButton.interactable = interactables;
		}

		public void SwitchOnOff()
		{
			if (onOffSprite)
			{
				imageButton.sprite = swithSprite[1];
				onOffSprite = false;
			}
			else
			{
				imageButton.sprite = swithSprite[0];
				onOffSprite = true;
			}
		}

		public void SwitchOnOff(bool on)
		{
			if(on)
			{
				imageButton.sprite = swithSprite[0];
				onOffSprite = true;
			}
			else
			{
				imageButton.sprite = swithSprite[1];
				onOffSprite = false;
			}
		}

		public void SetActive(bool enabled)
		{
			currentButton.gameObject.SetActive(enabled);
		}

		public void SetText(string text)
		{
			textButton.text = text;
		}

		public void SetImage(Sprite sp)
		{
			imageButton.sprite = sp;
		}

		public void SetSpriteIndex(int index)
		{
			if (swithSprite[index] == null) return;
			
			imageButton.sprite = swithSprite[index];
			
		}

		public void SetListner(UnityEngine.Events.UnityAction call)
		{
			currentButton.onClick.AddListener(call);
		}

		public void RemooveListners()
		{
			currentButton.onClick.RemoveAllListeners();
		}

		public void SwithSprite()
		{
			if (swithSprite.Count < 1) return;

			if (imageButton.sprite.name == swithSprite[0].name)
			{
				imageButton.sprite = swithSprite[1];
			}
			else
			{
				imageButton.sprite = swithSprite[0];
			}
		}

	}

}
