using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class MovePingPong : MonoBehaviour
{
	[SerializeField] private float speed = 2f;                                     //how fast to move
	[SerializeField] private float delay = 0f;                                     //how long to wait at each waypoint
	[SerializeField] private type movementType;                               //stop at final waypoint, loop through waypoints or move back n forth along waypoints
	[SerializeField] private List<Vector3> wayPoints = new List<Vector3>();
	public enum type { PlayOnce, Loop, PingPong }
	private int currentWp;
	private float arrivalTime;
    private bool forward = true, arrived = false;
	


	[ContextMenu("AddPosition")]
	public void AddPosition()
	{
		wayPoints.Add(transform.position);
	}


	void Update()
	{
		//if we've arrived at waypoint, get the next one
		if (wayPoints.Count > 0)
		{
			if (!arrived)
			{
				if (Vector3.Distance(transform.position, wayPoints[currentWp]) < 0.15f)
				{
					arrivalTime = Time.time;
					arrived = true;
				}
			}
			else
			{
				if (Time.time > arrivalTime + delay)
				{
					GetNextWP();
					arrived = false;
				}
			}
		}

	}

	void FixedUpdate()
	{
		if (!arrived && wayPoints.Count > 0)
		{
			Vector3 directionOfTravel = wayPoints[currentWp] - transform.position;
			directionOfTravel.Normalize();
			transform.Translate(
				directionOfTravel.x * speed * Time.deltaTime,
				directionOfTravel.y * speed * Time.deltaTime,
				directionOfTravel.z * speed * Time.deltaTime,
				Space.World
			);
			
		}

	}

	private void GetNextWP()
	{
		if (movementType == type.PlayOnce)
		{
			currentWp++;
			if (currentWp == wayPoints.Count)
				enabled = false;
		}

		if (movementType == type.Loop)
			currentWp = (currentWp == wayPoints.Count - 1) ? 0 : currentWp += 1;

		if (movementType == type.PingPong)
		{
			if (currentWp == wayPoints.Count - 1)
				forward = false;
			else if (currentWp == 0)
				forward = true;
			currentWp = (forward) ? currentWp += 1 : currentWp -= 1;
		}
	}
}












