using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using static SoundManager;

public class BallTrigger : MonoBehaviour
{
	[SerializeField] private Collider collider;
	[SerializeField] private GameObject explosionPref;
	[SerializeField] private Sound exolosionSound;
	[ColorUsage(true, true)]
	public Color colourDecal;
	[SerializeField] private float zPos = 9.3f;
	
	private GameManager gameManager;
	private PoolDecal poolDecal;

	private void Awake()
	{
		gameManager = GameManager.instance;
		poolDecal = gameManager.poolDecal;
	}

	private void Start()
	{
		if (collider == null)
		{
			collider = GetComponent<Collider>();
		}
		EventDispatcher.Add(EventNames.StartGame, () => 
		{ 
			collider.enabled = true; 
		});
		EventDispatcher.Add(EventNames.ResetedLevel, () => {
			collider.enabled = true;
			EndExplosion();
		});
		EventDispatcher.Add(EventNames.ResetingLevel, () => 
		{ 
			collider.enabled = false; 
		});
		EventDispatcher.Add(EventNames.WinGame, EndExplosion);
		EventDispatcher.Add(EventNames.ExitPlay, EndExplosion);
	}

	private void OnTriggerEnter(Collider other)
	{
		Vector3 location = transform.position;
		Vector3 closestPoint = other.ClosestPoint(location);
		closestPoint.z = zPos;
		poolDecal.GetDecaled(closestPoint, colourDecal);
		StartExlosion();
		EventDispatcher.SendEvent(EventNames.LoseGame);
	}


	private void StartExlosion()
	{
		//disabled this
		if (explosionPref != null)
		{
			explosionPref.transform.parent = null;
			explosionPref.SetActive(true);
			PlaySound(exolosionSound);
		}
		//explosionPrefab.transform.position 
	}

	private void EndExplosion()
	{
		if (explosionPref != null)
		{
			explosionPref.transform.parent = transform;
			explosionPref.transform.localPosition = Vector3.zero;
			explosionPref.SetActive(false);
		}
	}

}
