using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using static SoundManager;

public class PlayerMoove : MonoBehaviour
{
	[SerializeField] private Transform playerRotate;
	[SerializeField] private SpriteRenderer spriteInYang;
	[SerializeField] private GameObject centerParticle;
	[SerializeField] private GameObject aroundParticle;

	[SerializeField] private float coefResetSpeed = 1.3f; // by how much the return speed has increased relative to the distance for finishing.   15 
	[SerializeField] private float minDistance = 15f;
	[SerializeField] private float speedRotateReset = 8f;


	private ProgressLevel progressLevel;
	private Quaternion startRotate;
	private float resetSpeed;
	private bool startGame = false;
	private bool resetPlayer = false;
	private bool winGame = false;
	private float distanceToFinish = 15f;
	private float speedPlayer = 4f;
	private float speedRotate = 8;

	private bool noPaused = true;

	private void Start()
	{
		progressLevel = GameManager.instance.progress;
		startRotate = playerRotate.rotation;

		EventDispatcher.Add(EventNames.StartGame, StartGame);
		EventDispatcher.Add(EventNames.LoseGame, LoseGame);
		EventDispatcher.Add(EventNames.ExitPlay, ExitPlay);
		EventDispatcher.Add(EventNames.OpenMainMenu, StartRotation);
		EventDispatcher.Add(EventNames.OpenLevelMenu, StartRotation);
		EventDispatcher.Add(EventNames.Pause, () =>
		{
			noPaused = false;
		});
		EventDispatcher.Add(EventNames.UnPause, () =>
		{
			noPaused = true;
		});
		//EventDispatcher.Add(EventNames.p)
	}

	private void ExitPlay()
	{
		startGame = false;
		resetPlayer = false;
		winGame = false;
		transform.position = Vector3.zero;
		playerRotate.rotation = Quaternion.AngleAxis(-90, Vector3.forward);
	}

	private void StartRotation()
	{
		noPaused = true;
		startGame = false;
		resetPlayer = false;
		winGame = false;
		centerParticle.SetActive(false);
		aroundParticle.SetActive(true);
		playerRotate.rotation = startRotate;
		spriteInYang.enabled = true;
	}

	private void LoseGame()
	{
		startGame = false;
		resetPlayer = true;
		winGame = false;
		EventDispatcher.SendEvent(EventNames.ResetingLevel);
	}

	private void StartGame(object[] parametrs)
	{
		distanceToFinish = (float)parametrs[0];
		speedPlayer = (float)parametrs[1];
		speedRotate = (float)parametrs[2];
		resetSpeed = distanceToFinish * coefResetSpeed / minDistance;

		startGame = true;
		spriteInYang.enabled = false;
		playerRotate.rotation = Quaternion.AngleAxis(-90, Vector3.forward);
		centerParticle.SetActive(true);
		aroundParticle.SetActive(false);

		progressLevel.StartGame(distanceToFinish);
	}

//#if UNITY_EDITOR
//	private void FixedUpdate()
//	{
//		if (noPaused)
//		{
//			if (startGame)
//			{
//				if (Input.GetKey(KeyCode.D))
//				{
//					PointerRight();
//				}
//				else if (Input.GetKey(KeyCode.A))
//				{
//					PointerLeft();
//				}
//			}
//		}
//	}
//#endif

	private void Update()
	{
		if (noPaused)
		{
			if (startGame)
			{

				//#if UNITY_EDITOR
				//				if (Input.GetKey(KeyCode.D))
				//				{
				//					PointerRight();
				//				}
				//				else if (Input.GetKey(KeyCode.A))
				//				{
				//					PointerLeft();
				//				}
				//#endif

				if (Input.GetKey(KeyCode.D))
				{
					PointerRight();
				}
				else if (Input.GetKey(KeyCode.A))
				{
					PointerLeft();
				}


				TransformPlayer(Vector3.up);

				progressLevel.SetProgress(transform.position.y);

				if (transform.position.y >= distanceToFinish)
				{
					startGame = false;
					resetPlayer = true;
					winGame = true;
					PlaySound(Sound.WinGame);
					EventDispatcher.SendEvent(EventNames.TextWin);
					EventDispatcher.SendEvent(EventNames.ResetingLevel);
					progressLevel.EndGame();
				}
			}
			if (resetPlayer)
			{
				if (transform.position.y >= 0)
				{
					TransformPlayer(Vector3.down * resetSpeed);
					RotatePlayer(speedRotateReset);
				}
				else
				{
					resetPlayer = false;
					transform.position = Vector3.zero;
					playerRotate.rotation = Quaternion.AngleAxis(-90, Vector3.forward);
					if (winGame)
					{
						EventDispatcher.SendEvent(EventNames.WinGame);
						progressLevel.ResetFillAmount();
					}
					else
					{
						startGame = true;
						EventDispatcher.SendEvent(EventNames.ResetedLevel);
						progressLevel.ResetFillAmount();
					}
				}
			}
		}

	}

	private void TransformPlayer(Vector3 transformVector)
	{
		transform.Translate(transformVector * speedPlayer * Time.deltaTime);
	}

	private void RotatePlayer(float speed)
	{
		var rotate = Quaternion.AngleAxis(speed, Vector3.forward);
		playerRotate.rotation *= rotate;
	}

	public void PointerLeft()
	{
		if (startGame)
		{
			RotatePlayer(speedRotate);
		}
	}

	public void PointerRight()
	{
		if (startGame)
		{
			RotatePlayer(-speedRotate);
		}
	}
}
